@@ -16,6 +16,7 @@
 """This module contains the TestResult class, each instance of which holds status information for a single test method."""
 __testify = 1
 import datetime
+import sys
 import time
 import traceback
 
@@ -39,23 +40,73 @@ try:
 except ImportError:
     fancy_tb_formatter = None
 
+def plain_tb_formatter(etype, value, tb, length=None):
+    # We want our formatters to return a string.
+    return ''.join(traceback.format_exception(etype, value, tb, length))
 
 class TestResult(object):
+    debug = False
+
     def __init__(self, test_method, runner_id=None):
         super(TestResult, self).__init__()
         self.test_method = test_method
         self.test_method_name = test_method.__name__
         self.success = self.failure = self.error = self.interrupted = None
         self.run_time = self.start_time = self.end_time = None
-        self.exception_info = None
+        self.exception_infos = []
         self.complete = False
         self.previous_run = None
         self.runner_id = runner_id
 
+    @property
+    def exception_info(self):
+        raise AttributeError('The exception_info attribute has been replaced with the .exception_infos list. Please update your code.')
+
     def start(self, previous_run=None):
         self.previous_run = previous_run
         self.start_time = datetime.datetime.now()
 
+    def record(self, function):
+        """Excerpted code for executing a block of code that might raise an
+        exception, requiring us to update a result object.
+
+        Return value is a boolean describing whether the block was successfully
+        executed without exceptions.
+        """
+        try:
+            function()
+        except (KeyboardInterrupt, SystemExit):
+            raise
+        except Exception, exception:
+            # some code may want to use an alternative exc_info for an exception
+            # (for instance, in an event loop). You can signal an alternative
+            # stack to use by adding a _testify_exc_tb attribute to the
+            # exception object
+            if hasattr(exception, '_testify_exc_tb'):
+                exc_info = (type(exception), exception, exception._testify_exc_tb)
+            else:
+                exc_info = sys.exc_info()
+
+            self.end_in_failure(exc_info)
+
+            if self.debug:
+                self._postmortem(exc_info)
+            return False
+        else:
+            return True
+
+    def _postmortem(self, exc_info):
+        _, _, traceback = exc_info
+        print "\nDEBUGGER"
+        print self.format_exception_info()
+        try:
+            import ipdb
+            detected_postmortem_tool = ipdb.post_mortem
+        except ImportError:
+            import pdb
+            detected_postmortem_tool = pdb.post_mortem
+        detected_postmortem_tool(traceback)
+
     def _complete(self):
         self.complete = True
         self.end_time = datetime.datetime.now()
@@ -64,16 +115,16 @@ class TestResult(object):
     def end_in_failure(self, exception_info):
         if not self.complete:
             self._complete()
-        self.success = False
-        self.failure = True
-        self.exception_info = exception_info
 
-    def end_in_error(self, exception_info):
-        if not self.complete:
-            self._complete()
         self.success = False
-        self.error = True
-        self.exception_info = exception_info
+
+        if isinstance(exception_info[1], AssertionError):
+            # test failure, kinda expect these vs. unknown errors
+            self.failure = True
+        else:
+            self.error = True
+
+        self.exception_infos.append(exception_info)
 
     def end_in_success(self):
         if not self.complete:
@@ -84,39 +135,67 @@ class TestResult(object):
         if not self.complete:
             self._complete()
             self.interrupted = True
-            self.exception_info = exception_info
+            self.exception_infos.append(exception_info)
 
+    def __make_multi_error_message(self, formatter):
+        result = []
+        for exception_info in self.exception_infos:
+            exctype, value, tb = exception_info
+            part = formatter(exctype, value, tb)
+            result.append(part)
+
+        if len(result) == 1:
+            return result[0]
+        else:
+            # Meant to match the python3 multiple-exception support:
+            #   http://docs.python.org/3.1/reference/simple_stmts.html#the-raise-statement
+            return '\nDuring handling of the above exception, another exception occurred:\n\n'.join(result)
 
     def format_exception_info(self, pretty=False):
-        if self.exception_info is None:
+        if not self.exception_infos:
             return None
 
-        tb_formatter = fancy_tb_formatter if (pretty and fancy_tb_formatter) else traceback.format_exception
+        tb_formatter = fancy_tb_formatter if (pretty and fancy_tb_formatter) else plain_tb_formatter
 
         def is_relevant_tb_level(tb):
-            return tb.tb_frame.f_globals.has_key('__testify')
+            if tb.tb_frame.f_globals.has_key('__testify'):
+                # nobody *wants* to read testify
+                return False
+            else:
+                return True
 
         def count_relevant_tb_levels(tb):
+            # count up to the *innermost* relevant frame
             length = 0
-            while tb and not is_relevant_tb_level(tb):
+            relevant = 0
+            while tb:
                 length += 1
+                if is_relevant_tb_level(tb):
+                    relevant = length
                 tb = tb.tb_next
-            return length
+            return relevant
 
-        exctype, value, tb = self.exception_info
+        def formatter(exctype, value, tb):
+            # Skip test runner traceback levels at the top.
+            while tb and not is_relevant_tb_level(tb):
+                tb = tb.tb_next
 
-        # Skip test runner traceback levels
-        while tb and is_relevant_tb_level(tb):
-            tb = tb.tb_next
-        if exctype is AssertionError:
-            # Skip testify.assertions traceback levels
-            length = count_relevant_tb_levels(tb)
-            return tb_formatter(exctype, value, tb, length)
+            if exctype is AssertionError:
+                # Skip testify.assertions traceback levels at the bottom.
+                length = count_relevant_tb_levels(tb)
+                return tb_formatter(exctype, value, tb, length)
+            elif not tb:
+                return "Exception: %r (%r)" % (exctype, value)
+            else:
+                return tb_formatter(exctype, value, tb)
 
-        if not tb:
-            return "Exception: %r (%r)" % (exctype, value)
+        return self.__make_multi_error_message(formatter)
 
-        return tb_formatter(exctype, value, tb)
+    def format_exception_only(self):
+        def formatter(exctype, value, tb):
+            return ''.join(traceback.format_exception_only(exctype, value))
+
+        return self.__make_multi_error_message(formatter)
 
     def to_dict(self):
         return {
@@ -132,6 +211,7 @@ class TestResult(object):
             'interrupted' : self.interrupted,
             'exception_info' : self.format_exception_info(),
             'exception_info_pretty' : self.format_exception_info(pretty=True),
+            'exception_only' : self.format_exception_only(),
             'runner_id' : self.runner_id,
             'method' : {
                 'module' : self.test_method.im_class.__module__,
